/*
ScheduleTogether Backend
Copyright (C) 2024, Zervó Zadachin

DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License version 3
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License version 3 for more details.

This program incorporates external libraries for certain functionalities.
These libraries are covered by their respective licenses, and their usage
agreements are as outlined in their respective documentation or source
code.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package accounts

import (
	"github.com/kataras/iris/v12"

	"git.zervo.org/scheduletogether/backend/pkg/types"
)

// AccountSignout_Service signs out a user by invalidating the jwt token
func AccountSignout_Service(ctx iris.Context) {
	// Invalidate the token using the obtained claims
	err := ctx.Logout()
	if err != nil {
		ctx.StatusCode(iris.StatusInternalServerError)
		ctx.JSON(types.CommonErrorResponse{
			Error: "An error occured while signing out: " + err.Error(),
		})
		logboi.Warn("Couldn't sign out user: " + err.Error())
		return
	}

	ctx.JSON(types.CommonMessageResponse{
		Message: "Successfully signed out",
	})
}
