/*
ScheduleTogether Backend
Copyright (C) 2024, Zervó Zadachin

DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License version 3
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License version 3 for more details.

This program incorporates external libraries for certain functionalities.
These libraries are covered by their respective licenses, and their usage
agreements are as outlined in their respective documentation or source
code.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package types

import "github.com/kataras/iris/v12/middleware/jwt"

// Config holds configuration values for the application read from environment variables
type Config struct {
	ServerPort      int    `json:"serverPort"`
	DatabaseURL     string `json:"databaseURL"`
	DatabaseDialect string `json:"databaseDialect"`
	DatabaseDebug   bool   `json:"databaseDebug"`
	JWTSecret       string `json:"jwtSecret"`
	JWTExpiration   int    `json:"jwtExpiration"`
}

type Role string

type Claims struct {
	jwt.Claims        // Embedding the jwt.Claims standard properties
	UserId     uint   `json:"user_id"`
	Role       string `json:"role"`
	Username   string `json:"username"`
	Email      string `json:"email"`
	Token      string `json:"token"`
}

type PermissionKey int

type Permission struct {
	Key         string   `json:"key"`
	Name        string   `json:"name"`
	Description string   `json:"description"`
	Roles       []string `json:"roles"`
}

type FriendRequest struct {
	CreatedAt      string `json:"created_at"`
	SourceUsername string `json:"source_username"`
	SourceUuid     string `json:"source_uuid"`
	TargetUuid     string `json:"target_uuid"`
}
